# -*- coding: utf-8 -*-

"""The graphical part of a Gaussian Energy node"""

import logging
import tkinter as tk
import tkinter.ttk as ttk

import gaussian_step
import seamm
import seamm_widgets as sw

logger = logging.getLogger("Gaussian")


class TkEnergy(seamm.TkNode):
    def __init__(
        self,
        tk_flowchart=None,
        node=None,
        canvas=None,
        x=120,
        y=20,
        w=200,
        h=50,
        my_logger=logger,
    ):
        """Initialize the graphical Tk Gaussian energy step

        Keyword arguments:
        """
        self.results_widgets = []

        super().__init__(
            tk_flowchart=tk_flowchart,
            node=node,
            canvas=canvas,
            x=x,
            y=y,
            w=w,
            h=h,
            my_logger=my_logger,
        )

    def right_click(self, event):
        """Probably need to add our dialog..."""

        super().right_click(event)
        self.popup_menu.add_command(label="Edit..", command=self.edit)

        self.popup_menu.tk_popup(event.x_root, event.y_root, 0)

    def create_dialog(self, title="Edit Gaussian Energy Step"):
        """Create the dialog!"""
        self.logger.debug("Creating the dialog")
        frame = super().create_dialog(title=title, widget="notebook", results_tab=True)

        # Create a frame for the calculation control
        self["calculation"] = ttk.LabelFrame(
            frame,
            borderwidth=4,
            relief="sunken",
            text="Calculation",
            labelanchor="n",
            padding=10,
        )
        # Create a frame for the convergence control
        self["convergence frame"] = ttk.LabelFrame(
            frame,
            borderwidth=4,
            relief="sunken",
            text="SCF Convergence Control",
            labelanchor="n",
            padding=10,
        )

        # Create all the widgets
        P = self.node.parameters
        for key in (
            "level",
            "method",
            "basis",
            "advanced_method",
            "functional",
            "advanced_functional",
            "dispersion",
            "spin-restricted",
            "freeze-cores",
        ):
            self[key] = P[key].widget(self["calculation"])

        # bindings...
        for key in ("level", "method", "advanced_method"):
            self[key].bind("<<ComboboxSelected>>", self.reset_calculation)
            self[key].bind("<Return>", self.reset_calculation)
            self[key].bind("<FocusOut>", self.reset_calculation)

        for key in (
            "maximum iterations",
            "convergence",
        ):
            self[key] = P[key].widget(self["convergence frame"])

        # Create the structure-handling widgets
        sframe = self["structure frame"] = ttk.LabelFrame(
            frame, text="Configuration Handling", labelanchor=tk.N
        )
        row = 0
        widgets = []
        for key in ("structure handling", "system name", "configuration name"):
            self[key] = P[key].widget(sframe)
            self[key].grid(row=row, column=0, sticky=tk.EW)
            widgets.append(self[key])
            row += 1
        sw.align_labels(widgets, sticky=tk.E)

        # A tab for output of orbitals, etc.
        notebook = self["notebook"]
        self["output frame"] = oframe = ttk.Frame(notebook)
        notebook.insert(self["results frame"], oframe, text="Output", sticky="new")

        # Frame to isolate widgets
        p_frame = self["plot frame"] = ttk.LabelFrame(
            self["output frame"],
            borderwidth=4,
            relief="sunken",
            text="Plots",
            labelanchor="n",
            padding=10,
        )

        for key in gaussian_step.EnergyParameters.output_parameters:
            self[key] = P[key].widget(p_frame)

        # Set the callbacks for changes
        for widget in ("orbitals", "region"):
            w = self[widget]
            w.combobox.bind("<<ComboboxSelected>>", self.reset_plotting)
            w.combobox.bind("<Return>", self.reset_plotting)
            w.combobox.bind("<FocusOut>", self.reset_plotting)
        p_frame.grid(row=0, column=0, sticky="new")
        oframe.columnconfigure(0, weight=1)

        # and lay them out
        self.reset_plotting()

        # Top level needs to call reset_dialog
        if self.node.calculation == "energy":
            self.reset_dialog()

        self.logger.debug("Finished creating the dialog")

    def reset_dialog(self, widget=None):
        """Layout the widgets as needed for the current state"""

        frame = self["frame"]
        for slave in frame.grid_slaves():
            slave.grid_forget()

        row = 0
        self["calculation"].grid(row=row, column=0)
        row += 1
        self.reset_calculation()
        self["convergence frame"].grid(row=row, column=0)
        row += 1
        self.reset_convergence()
        self["structure frame"].grid(
            row=row, column=0, columnspan=2, sticky=tk.N, pady=5
        )
        row += 1
        return row

    def reset_calculation(self, widget=None):
        level = self["level"].get()

        if level == "recommended":
            long_method = self["method"].get()
            functional = self["functional"].get()
            widget = self["method"]
        else:
            long_method = self["advanced_method"].get()
            functional = self["advanced_functional"].get()
            widget = self["advanced_method"]
        if self.is_expr(long_method):
            self.node.method = None
            meta = None
        else:
            if long_method in gaussian_step.methods:
                self.node.method = gaussian_step.methods[long_method]["method"]
                meta = gaussian_step.methods[long_method]
            else:
                # See if it matches the keyword part
                for key, mdata in gaussian_step.methods.items():
                    if long_method == mdata["method"]:
                        long_method = key
                        widget.set(long_method)
                        meta = mdata
                        self.node.method = meta["method"]
                        break
                else:
                    self.node.method = long_method
                    meta = None

        # Set up the results table because it depends on the method
        self.results_widgets = []
        self.setup_results()

        frame = self["calculation"]
        for slave in frame.grid_slaves():
            slave.grid_forget()

        widgets = []
        widgets2 = []
        row = 0
        self["level"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
        row += 1
        if level == "recommended":
            self["method"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
            widgets.append(self["method"])
            row += 1
            if meta is None or "nobasis" not in meta or not meta["nobasis"]:
                self["basis"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
                widgets.append(self["basis"])
                row += 1
            if self.node.method is None or self.node.method == "DFT":
                self["functional"].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self["functional"])
                row += 1
            if meta is None or "freeze core?" in meta and meta["freeze core?"]:
                self["freeze-cores"].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self["freeze-cores"])
                row += 1
        else:
            self["advanced_method"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
            widgets.append(self["advanced_method"])
            row += 1
            if meta is None or "nobasis" not in meta or not meta["nobasis"]:
                self["basis"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
                widgets.append(self["basis"])
                row += 1
            if self.node.method is None or self.node.method == "DFT":
                self["advanced_functional"].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self["advanced_functional"])
                row += 1
            if meta is None or "freeze core?" in meta and meta["freeze core?"]:
                self["freeze-cores"].grid(row=row, column=1, sticky=tk.EW)
                widgets2.append(self["freeze-cores"])
                row += 1
        if self.node.method is None or self.node.method == "DFT":
            if functional in gaussian_step.dft_functionals:
                dispersions = gaussian_step.dft_functionals[functional]["dispersion"]
                if len(dispersions) > 1:
                    w = self["dispersion"]
                    w.config(values=dispersions)
                    if w.get() not in dispersions:
                        w.value(dispersions[1])
                    w.grid(row=row, column=1, sticky=tk.W)
                    widgets2.append(self["dispersion"])
                    row += 1
                    sw.align_labels(widgets2, sticky=tk.E)
                frame.columnconfigure(0, minsize=30)
        self["spin-restricted"].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
        widgets.append(self["spin-restricted"])
        row += 1
        sw.align_labels(widgets, sticky=tk.E)

        return row

    def reset_convergence(self, widget=None):
        """Layout the convergence widgets as needed for the current state"""

        frame = self["convergence frame"]
        for slave in frame.grid_slaves():
            slave.grid_forget()

        widgets = []
        row = 0

        for key in (
            "maximum iterations",
            "convergence",
        ):
            self[key].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
            widgets.append(self[key])
            row += 1

        frame.columnconfigure(0, minsize=150)
        sw.align_labels(widgets, sticky=tk.E)

    def reset_plotting(self, widget=None):
        frame = self["plot frame"]
        for slave in frame.grid_slaves():
            slave.grid_forget()

        plot_orbitals = self["orbitals"].get() == "yes"
        region = self["region"].get()

        widgets = []

        row = 0
        for key in (
            "total density",
            "total spin density",
            "orbitals",
        ):
            # "difference density",
            self[key].grid(row=row, column=0, columnspan=4, sticky=tk.EW)
            widgets.append(self[key])
            row += 1

        if plot_orbitals:
            key = "selected orbitals"
            self[key].grid(row=row, column=1, columnspan=4, sticky=tk.EW)
            row += 1

        key = "region"
        self[key].grid(row=row, column=0, columnspan=4, sticky=tk.EW)
        widgets.append(self[key])
        row += 1

        if region == "explicit":
            key = "nx"
            self[key].grid(row=row, column=0, columnspan=2, sticky=tk.EW)
            widgets.append(self[key])
            self["ny"].grid(row=row, column=2, sticky=tk.EW)
            self["nz"].grid(row=row, column=3, sticky=tk.EW)

        sw.align_labels(widgets, sticky=tk.E)
        frame.columnconfigure(0, minsize=10)
        frame.columnconfigure(4, weight=1)

        return row



# -*- coding: utf-8 -*-

import gaussian_step


class GaussianStep(object):
    """Helper class needed for the stevedore integration.

    This must provide a description() method that returns a dict containing a
    description of this node, and create_node() and create_tk_node() methods
    for creating the graphical and non-graphical nodes.

    Parameters
    ----------
    my_description : {description, group, name}
        A human-readable description of this step. It can be
        several lines long, and needs to be clear to non-expert users.
        It contains the following keys: description, group, name.
    my_description["description"] : tuple
        A description of the Gaussian step. It must be
        clear to non-experts.
    my_description["group"] : str
        Which group in the menus to put this step. If the group does
        not exist it will be created. Common groups are "Building",
        "Calculations", "Control" and "Data".
    my_description["name"] : str
        The name of this step, to be displayed in the menus.
    """

    my_description = {
        "description": "An interface for Gaussian",
        "group": "Simulations",
        "name": "Gaussian",
    }

    def __init__(self, flowchart=None, gui=None):
        """Initialize this helper class, which is used by
        the application via stevedore to get information about
        and create node objects for the flowchart
        """
        pass

    def description(self):
        """Return a description of what this extension does."""
        return GaussianStep.my_description

    def create_node(self, flowchart=None, **kwargs):
        """Create and return the new node object.

        Parameters
        ----------
        flowchart: seamm.Node
            A non-graphical SEAMM node

        **kwargs : keyworded arguments
            Various keyworded arguments such as title, namespace or
            extension representing the title displayed in the flowchart,
            the namespace for the plugins of a subflowchart and
            the extension, respectively.

        Returns
        -------
        Gaussian

        See Also
        --------
        Gaussian

        """

        return gaussian_step.Gaussian(flowchart=flowchart, **kwargs)

    def create_tk_node(self, canvas=None, **kwargs):
        """Create and return the graphical Tk node object.

        Parameters
        ----------
        canvas : tk.Canvas
            The Tk Canvas widget

        **kwargs : keyworded arguments
            Various keyworded arguments such as tk_flowchart, node, x, y, w, h
            representing a graphical flowchart object, a non-graphical node for
            a step, and dimensions of the graphical node.

        Returns
        -------
        TkGaussian

        See Also
        --------
        TkGaussian
        """

        return gaussian_step.TkGaussian(canvas=canvas, **kwargs)



# -*- coding: utf-8 -*-

"""Setup and run Gaussian"""

import gzip
import logging
from pathlib import Path
import pprint
import re
import shutil
import string
import subprocess

import cclib
import numpy as np
import psutil

import gaussian_step
import seamm
import seamm.data
import seamm_util.printing as printing

logger = logging.getLogger("Gaussian")
job = printing.getPrinter()
printer = printing.getPrinter("gaussian")


def humanize(memory, suffix="B", kilo=1024):
    """
    Scale memory to its proper format e.g:

        1253656 => '1.20 MiB'
        1253656678 => '1.17 GiB'
    """
    if kilo == 1000:
        units = ["", "k", "M", "G", "T", "P"]
    elif kilo == 1024:
        units = ["", "Ki", "Mi", "Gi", "Ti", "Pi"]
    else:
        raise ValueError("kilo must be 1000 or 1024!")

    for unit in units:
        if memory < 10 * kilo:
            return f"{int(memory)}{unit}{suffix}"
        memory /= kilo


def dehumanize(memory, suffix="B"):
    """
    Unscale memory from its human readable form e.g:

        '1.20 MB' => 1200000
        '1.17 GB' => 1170000000
    """
    units = {
        "": 1,
        "k": 1000,
        "M": 1000**2,
        "G": 1000**3,
        "P": 1000**4,
        "Ki": 1024,
        "Mi": 1024**2,
        "Gi": 1024**3,
        "Pi": 1024**4,
    }

    tmp = memory.split()
    if len(tmp) == 1:
        return memory
    elif len(tmp) > 2:
        raise ValueError("Memory must be <number> <units>, e.g. 1.23 GB")

    amount, unit = tmp
    amount = float(amount)

    for prefix in units:
        if prefix + suffix == unit:
            return int(amount * units[prefix])

    raise ValueError(f"Don't recognize the units on '{memory}'")


class Substep(seamm.Node):
    def __init__(
        self,
        flowchart=None,
        title="no title",
        extension=None,
        logger=logger,
        module=__name__,
    ):
        """Initialize the node"""

        logger.debug("Creating Energy {}".format(self))

        super().__init__(
            flowchart=flowchart, title=title, extension=extension, logger=logger
        )

    @property
    def version(self):
        """The semantic version of this module."""
        return gaussian_step.__version__

    @property
    def git_revision(self):
        """The git version of this module."""
        return gaussian_step.__git_revision__

    @property
    def global_options(self):
        return self.parent.global_options

    @property
    def is_runable(self):
        """Indicate whether this not runs or just adds input."""
        return True

    @property
    def method(self):
        """The method ... HF, DFT, ... used."""
        return self._method

    @method.setter
    def method(self, value):
        self._method = value

    @property
    def options(self):
        return self.parent.options

    def make_plots(self, data):
        """Create the density and orbital plots if requested.

        Parameters
        ----------
        data : dict()
             Dictionary of results from the calculation (results.tag file)
        """
        text = "\n\n"

        P = self.parameters.current_values_to_dict(
            context=seamm.flowchart_variables._data
        )

        # Get the configuration and basic information
        system, configuration = self.get_system_configuration(None)

        periodicity = configuration.periodicity
        if periodicity != 0:
            raise NotImplementedError("Periodic cube files not implemented yet!")
        spin_polarized = len(data["homos"]) == 2

        # Read the detailed output file to get the number of iterations
        directory = Path(self.directory)

        if self.options["gaussian_root"] != "":
            env = {"g09root": self.options["gaussian_root"]}
        else:
            env = {}

        if self.options["gaussian_environment"] != "":
            cmd = f". {self.options['gaussian_environment']} && cubegen"
        else:
            cmd = "cubegen"

        npts = "-2"

        keys = []
        if P["total density"]:
            keys.append("total density")
        if spin_polarized and P["total spin density"]:
            keys.append("spin density")

        n_errors = 0
        for key in keys:
            if key == "total density":
                args = f"1 Density=SCF gaussian.fchk Total_Density.cube {npts} h"
            elif key == "spin density":
                args = f"1 Spin=SCF gaussian.fchk Spin_Density.cube {npts} h"

            # And run CUBEGEN
            try:
                output = subprocess.check_output(
                    cmd + " " + args,
                    shell=True,
                    text=True,
                    env=env,
                    stderr=subprocess.STDOUT,
                    cwd=directory,
                )
                logger.debug(f"Output from CUBEGEN:\n{output}")
            except subprocess.CalledProcessError as e:
                n_errors += 1
                printer.important(
                    f"Calling CUBEGEN, {cmd} {args}:"
                    f"returncode = {e.returncode}\n\nOutput: {e.output}"
                )

        # Any requested orbitals
        if P["orbitals"]:
            n_orbitals = data["nmo"]
            # and work out the orbitals
            txt = P["selected orbitals"]
            for spin, homo in enumerate(data["homos"]):
                if txt == "all":
                    orbitals = [*range(n_orbitals)]
                else:
                    orbitals = []
                    for chunk in txt.split(","):
                        chunk = chunk.strip()
                        if ":" in chunk or ".." in chunk:
                            if ":" in chunk:
                                first, last = chunk.split(":")
                            elif ".." in chunk:
                                first, last = chunk.split("..")
                            first = first.strip().upper()
                            last = last.strip().upper()

                            if first == "HOMO":
                                first = homo
                            elif first == "LUMO":
                                first = homo + 1
                            else:
                                first = int(
                                    first.removeprefix("HOMO").removeprefix("LUMO")
                                )
                                if first < 0:
                                    first = homo + first
                                else:
                                    first = homo + 1 + first

                            if last == "HOMO":
                                last = homo
                            elif last == "LUMO":
                                last = homo + 1
                            else:
                                last = int(
                                    last.removeprefix("HOMO").removeprefix("LUMO")
                                )
                                if last < 0:
                                    last = homo + last
                                else:
                                    last = homo + 1 + last

                            orbitals.extend(range(first, last + 1))
                        else:
                            first = chunk.strip().upper()

                            if first == "HOMO":
                                first = homo
                            elif first == "LUMO":
                                first = homo + 1
                            else:
                                first = int(
                                    first.removeprefix("HOMO").removeprefix("LUMO")
                                )
                                if first < 0:
                                    first = homo + first
                                else:
                                    first = homo + 1 + first
                            orbitals.append(first)

                # Remove orbitals out of limits
                tmp = orbitals
                orbitals = []
                for x in tmp:
                    if x >= 0 and x < n_orbitals:
                        orbitals.append(x)

                if spin_polarized:
                    l1 = ("A", "B")[spin]
                    l2 = ("α-", "β-")[spin]
                else:
                    l1 = ""
                    l2 = ""
                for mo in orbitals:
                    if mo == homo:
                        filename = f"{l2}HOMO.cube"
                    elif mo < homo:
                        filename = f"{l2}HOMO-{homo - mo}.cube"
                    elif mo == homo + 1:
                        filename = f"{l2}LUMO.cube"
                    else:
                        filename = f"{l2}LUMO+{mo - homo - 1}.cube"
                    args = f"1 {l1}MO={mo + 1} gaussian.fchk {filename} {npts} h"

                    # And run CUBEGEN
                    try:
                        output = subprocess.check_output(
                            cmd + " " + args,
                            shell=True,
                            text=True,
                            env=env,
                            stderr=subprocess.STDOUT,
                            cwd=directory,
                        )
                        logger.debug(f"Output from CUBEGEN:\n{output}")
                    except subprocess.CalledProcessError as e:
                        n_errors += 1
                        printer.important(
                            f"Calling CUBEGEN, {cmd} {args}:"
                            f"returncode = {e.returncode}\n\nOutput: {e.output}"
                        )

        # Finally rename and gzip the cube files
        n_processed = 0
        paths = directory.glob("*.cube")
        for path in paths:
            out = path.with_suffix(".cube.gz")
            with path.open("rb") as f_in:
                with gzip.open(out, "wb") as f_out:
                    shutil.copyfileobj(f_in, f_out)
            n_processed += 1
            path.unlink()
        if n_errors > 0:
            text += (
                f"Created {n_processed} density and orbital cube files, but there were "
                f"{n_errors} errors trying to create cube files."
            )
        else:
            text += f"Created {n_processed} density and orbital cube files."

        return text

    def parse_fchk(self, path, data={}):
        """Process the data of a formatted Chk file given as lines of data.

        Parameters
        ----------
        path : pathlib.Path
            The path to the checkpoint file
        """
        lines = path.read_text().splitlines()

        it = iter(lines)
        # Ignore first potentially truncated title line
        next(it)

        # Type line (A10,A30,A30)
        line = next(it)
        data["calculation"] = line[0:10].strip()
        data["method"] = line[10:40].strip()
        data["basis"] = line[40:70].strip()

        # The rest of the file consists of a line defining the data.
        # If the data is a scalar, it is on the control line, otherwise it follows
        while True:
            try:
                line = next(it)
            except StopIteration:
                break
            key = line[0:40].strip()
            code = line[43]
            is_array = line[47:49] == "N="
            if is_array:
                count = int(line[49:61].strip())
                value = []
                if code == "I":
                    i = 0
                    while i < count:
                        line = next(it)
                        for pos in range(0, 6 * 12, 12):
                            value.append(int(line[pos : pos + 12].strip()))
                            i += 1
                            if i == count:
                                break
                elif code == "R":
                    i = 0
                    while i < count:
                        line = next(it)
                        for pos in range(0, 5 * 16, 16):
                            text = line[pos : pos + 16].strip()
                            # Fortran drops E in format for large exponents...
                            text = re.sub(r"([0-9])-", r"\1E-", text)
                            value.append(float(text))
                            i += 1
                            if i == count:
                                break
                elif code == "C":
                    value = ""
                    i = 0
                    while i < count:
                        line = next(it)
                        for pos in range(0, 5 * 12, 12):
                            value += line[pos : pos + 12]
                            i += 1
                            if i == count:
                                break
                    value = value.rstrip()
                elif code == "H":
                    value = ""
                    i = 0
                    while i < count:
                        line = next(it)
                        for pos in range(0, 9 * 8, 8):
                            value += line[pos : pos + 8]
                            i += 1
                            if i == count:
                                break
                    value = value.rstrip()
                elif code == "L":
                    i = 0
                    while i < count:
                        line = next(it)
                        for pos in range(72):
                            value.append(line[pos] == "T")
                            i += 1
                            if i == count:
                                break
            else:
                if code == "I":
                    value = int(line[49:].strip())
                elif code == "R":
                    value = float(line[49:].strip())
                elif code == "C":
                    value = line[49:].strip()
                elif code == "L":
                    value = line[49] == "T"
            data[key] = value
        return data

    def parse_output(self, path, data={}):
        """Process the output.

        Parameters
        ----------
        path : pathlib.Path
            The Gaussian log file.
        """
        P = self.parameters.current_values_to_dict(
            context=seamm.flowchart_variables._data
        )

        lines = path.read_text().splitlines()

        # Did it end properly?
        data["success"] = "Normal termination" in lines[-1]

        # Find the date and version of Gaussian
        # Gaussian 09:  EM64M-G09RevE.01 30-Nov-2015
        it = iter(lines)
        for line in it:
            if "Cite this work" in line:
                for line in it:
                    if "**********************" in line:
                        line = next(it)
                        if "Gaussian" in line:
                            try:
                                _, version, revision, date = line.split()
                                _, month, year = date.split("-")
                                revision = revision.split("Rev")[1]
                                data["G revision"] = revision
                                data["G version"] = f"G{version.strip(':')}"
                                data["G month"] = month
                                data["G year"] = year
                            except Exception as e:
                                logger.warning(
                                    f"Could not find the Gaussian citation: {e}"
                                )
                            break
                break

        # And the optimization steps, if any.
        it = iter(lines)
        n_steps = 0
        max_force = []
        rms_force = []
        max_displacement = []
        rms_displacement = []
        converged = None
        for line in it:
            if line == "         Item               Value     Threshold  Converged?":
                n_steps += 1
                converged = True

                tmp1, tmp2, value, threshold, criterion = next(it).split()
                if tmp1 == "Maximum" and tmp2 == "Force":
                    max_force.append(float(value))
                    data["Maximum Force Threshold"] = float(threshold)
                    if criterion != "YES":
                        converged = False

                tmp1, tmp2, value, threshold, criterion = next(it).split()
                if tmp1 == "RMS" and tmp2 == "Force":
                    rms_force.append(float(value))
                    data["RMS Force Threshold"] = float(threshold)
                    if criterion != "YES":
                        converged = False

                tmp1, tmp2, value, threshold, criterion = next(it).split()
                if tmp1 == "Maximum" and tmp2 == "Displacement":
                    max_displacement.append(float(value))
                    data["Maximum Displacement Threshold"] = float(threshold)
                    if criterion != "YES":
                        converged = False

                tmp1, tmp2, value, threshold, criterion = next(it).split()
                if tmp1 == "RMS" and tmp2 == "Displacement":
                    rms_displacement.append(float(value))
                    data["RMS Displacement Threshold"] = float(threshold)
                    if criterion != "YES":
                        converged = False

        if converged is not None:
            data["Geometry Optimization Converged"] = converged
            data["Maximum Force"] = max_force[-1]
            data["RMS Force"] = rms_force[-1]
            data["Maximum Displacement"] = max_displacement[-1]
            data["RMS Displacement"] = rms_displacement[-1]
            data["Maximum Force Trajectory"] = max_force
            data["RMS Force Trajectory"] = rms_force
            data["Maximum Displacement Trajectory"] = max_displacement
            data["RMS Displacement Trajectory"] = rms_displacement

        # CBS calculations

        # Complete Basis Set (CBS) Extrapolation:
        # M. R. Nyden and G. A. Petersson, JCP 75, 1843 (1981)
        # G. A. Petersson and M. A. Al-Laham, JCP 94, 6081 (1991)
        # G. A. Petersson, T. Tensfeldt, and J. A. Montgomery, JCP 94, 6091 (1991)
        # J. A. Montgomery, J. W. Ochterski, and G. A. Petersson, JCP 101, 5900 (1994)
        #
        # Temperature=               298.150000 Pressure=                       1.000000
        # E(ZPE)=                      0.050496 E(Thermal)=                     0.053508
        # E(SCF)=                    -78.059017 DE(MP2)=                       -0.281841
        # DE(CBS)=                    -0.071189 DE(MP34)=                      -0.024136
        # DE(Int)=                     0.021229 DE(Empirical)=                 -0.075463
        # CBS-4 (0 K)=               -78.439921 CBS-4 Energy=                 -78.436908
        # CBS-4 Enthalpy=            -78.435964 CBS-4 Free Energy=            -78.460753

        if P["method"][0:4] == "CBS-":
            # Need last section
            if P["method"] in gaussian_step.methods:
                method = gaussian_step.methods[P["method"]]["method"]
            else:
                method = P["method"]

            match = f"{method} Enthalpy="
            text = []
            found = False
            for line in reversed(lines):
                if found:
                    text.append(line)
                    if "Complete Basis Set" in line:
                        break
                elif match in line:
                    found = True
                    text.append(line)

            if found:
                text = text[::-1]
                it = iter(text)
                next(it)
                citations = []
                for line in it:
                    tmp = line.strip()
                    if tmp == "":
                        break
                    citations.append(tmp)
                data["citations"] = citations

                for line in it:
                    line = line.strip()
                    if len(line) > 40:
                        part = [line[0:37], line[38:]]
                    else:
                        part = [line]
                    for p in part:
                        if "=" not in p:
                            continue
                        key, value = p.split("=", 1)
                        key = key.strip()
                        value = float(value.strip())
                        if method in key:
                            key = key.split(" ", 1)[1]
                        data[f"Composite/{key}"] = value
                data["Composite/model"] = method
                data["Composite/summary"] = "\n".join(text)
                data["Total Energy"] = data["Composite/Free Energy"]

        # Gn calculations. No header!!!!!

        # Temperature=              298.150000 Pressure=                      1.000000
        # E(ZPE)=                     0.050251 E(Thermal)=                    0.053306
        # E(CCSD(T))=               -78.321715 E(Empiric)=                   -0.041682
        # DE(Plus)=                  -0.005930 DE(2DF)=                      -0.076980
        # E(Delta-G3XP)=             -0.117567 DE(HF)=                       -0.008255
        # G4(0 K)=                  -78.521880 G4 Energy=                   -78.518825
        # G4 Enthalpy=              -78.517880 G4 Free Energy=              -78.542752

        if P["method"][0:2] in ("G1", "G2", "G3", "G4"):
            # Need last section
            method = P["method"][0:2]
            match = f"{method} Enthalpy="
            text = []
            found = False
            for line in reversed(lines):
                if found:
                    if line.strip() == "":
                        break
                    text.append(line)
                elif match in line:
                    found = True
                    text.append(line)

            if found:
                text = text[::-1]
                for line in text:
                    line = line.strip()
                    if len(line) > 36:
                        part = [line[0:36], line[37:]]
                    else:
                        part = [line]
                    for p in part:
                        if "=" not in p:
                            continue
                        key, value = p.split("=", 1)
                        key = key.strip()
                        value = float(value.strip())
                        if method in key:
                            key = key.split(" ", 1)[1]
                        elif key == "E(Empiric)":
                            key = "E(empirical)"
                        data[f"Composite/{key}"] = value

                data["Composite/model"] = method
                tmp = " " * 20 + f"{method[0:2]} composite method extrapolation\n\n"
                data["Composite/summary"] = tmp + "\n".join(text)
                data["Total Energy"] = data["Composite/Free Energy"]

        return data

    def process_data(self, data):
        """Massage the cclib data to a more easily used form."""
        # Convert numpy arrays to Python lists
        new = {}
        for key, value in data.items():
            if isinstance(value, np.ndarray):
                new[key] = value.tolist()
            elif isinstance(value, list):
                if len(value) > 0 and isinstance(value[0], np.ndarray):
                    new[key] = [i.tolist() for i in value]
                else:
                    new[key] = value
            elif isinstance(value, dict):
                for k, v in value.items():
                    newkey = f"{key}/{k}"
                    if isinstance(v, np.ndarray):
                        new[newkey] = v.tolist()
                    else:
                        new[newkey] = v
            else:
                new[key] = value

        for key in ("metadata/cpu_time", "metadata/wall_time"):
            if key in new:
                time = new[key][0]
                for tmp in new[key][1:]:
                    time += tmp
                new[key] = str(time).lstrip("0:")
                if "." in new[key]:
                    new[key] = new[key].rstrip("0")

        # Pull out the HOMO and LUMO energies as scalars
        if "homos" in new and "moenergies" in new:
            homos = new["homos"]
            if len(homos) == 2:
                for i, letter in enumerate(["α", "β"]):
                    Es = new["moenergies"][i]
                    homo = homos[i]
                    new[f"N({letter}-homo)"] = homo + 1
                    new[f"E({letter}-homo)"] = Es[homo]
                    if homo > 0:
                        new[f"E({letter}-homo-1)"] = Es[homo - 1]
                    if homo + 1 < len(Es):
                        new[f"E({letter}-lumo)"] = Es[homo + 1]
                        new[f"E({letter}-gap)"] = Es[homo + 1] - Es[homo]
                    if homo + 2 < len(Es):
                        new[f"E({letter}-lumo+1)"] = Es[homo + 2]
                    if "mosyms" in new:
                        syms = new["mosyms"][i]
                        new[f"Sym({letter}-homo)"] = syms[homo]
                        if homo > 0:
                            new[f"Sym({letter}-homo-1)"] = syms[homo - 1]
                        if homo + 1 < len(syms):
                            new[f"Sym({letter}-lumo)"] = syms[homo + 1]
                        if homo + 2 < len(syms):
                            new[f"Sym({letter}-lumo+1)"] = syms[homo + 2]
            else:
                Es = new["moenergies"][0]
                homo = homos[0]
                new["N(homo)"] = homo + 1
                new["E(homo)"] = Es[homo]
                if homo > 0:
                    new["E(homo-1)"] = Es[homo - 1]
                if homo + 1 < len(Es):
                    new["E(lumo)"] = Es[homo + 1]
                    new["E(gap)"] = Es[homo + 1] - Es[homo]
                if homo + 2 < len(Es):
                    new["E(lumo+1)"] = Es[homo + 2]
                if "mosyms" in new:
                    syms = new["mosyms"][0]
                    new["Sym(homo)"] = syms[homo]
                    if homo > 0:
                        new["Sym(homo-1)"] = syms[homo - 1]
                    if homo + 1 < len(syms):
                        new["Sym(lumo)"] = syms[homo + 1]
                    if homo + 2 < len(syms):
                        new["Sym(lumo+1)"] = syms[homo + 2]

        # moments
        if "moments" in new:
            moments = new["moments"]
            new["multipole_reference"] = moments[0]
            new["dipole_moment"] = moments[1]
            new["dipole_moment_magnitude"] = np.linalg.norm(moments[1])
            if len(moments) > 2:
                new["quadrupole_moment"] = moments[2]
            if len(moments) > 3:
                new["octapole_moment"] = moments[3]
            if len(moments) > 4:
                new["hexadecapole_moment"] = moments[4]
            del new["moments"]

        for key in ("metadata/symmetry_detected", "metadata/symmetry_used"):
            if key in new:
                new[key] = new[key].capitalize()

        return new

    def run_gaussian(self, keywords):
        """Run Gaussian.

        Parameters
        ----------
        None

        Returns
        -------
        seamm.Node
            The next node object in the flowchart.
        """
        # Create the directory
        directory = Path(self.directory)
        directory.mkdir(parents=True, exist_ok=True)

        # Check for successful run, don't rerun
        success = directory / "success.dat"
        if not success.exists():
            # Get the system & configuration
            system, configuration = self.get_system_configuration(None)

            # Access the options
            options = self.options
            seamm_options = self.global_options

            # Work out how many cores and how much memory to use
            n_cores = psutil.cpu_count(logical=False)
            self.logger.info("The number of cores is {}".format(n_cores))

            # How many threads to use
            if seamm_options["parallelism"] not in ("openmp", "any"):
                n_threads = 1
            else:
                if options["ncores"] == "available":
                    n_threads = n_cores
                else:
                    n_threads = int(options["ncores"])
                if n_threads > n_cores:
                    n_threads = n_cores
                if n_threads < 1:
                    n_threads = 1
                if seamm_options["ncores"] != "available":
                    n_threads = min(n_threads, int(seamm_options["ncores"]))
            self.logger.info(f"Gaussian will use {n_threads} threads.")

            # How much memory to use
            svmem = psutil.virtual_memory()

            if seamm_options["memory"] == "all":
                mem_limit = svmem.total
            elif seamm_options["memory"] == "available":
                # For the default, 'available', use in proportion to number of
                # cores used
                mem_limit = svmem.total * (n_threads / n_cores)
            else:
                mem_limit = dehumanize(seamm_options["memory"])

            if options["memory"] == "all":
                memory = svmem.total
            elif options["memory"] == "available":
                # For the default, 'available', use in proportion to number of
                # cores used
                memory = svmem.total * (n_threads / n_cores)
            else:
                memory = dehumanize(options["memory"])

            memory = min(memory, mem_limit)

            # Apply a minimum of 800 MB
            min_memory = dehumanize("800 MB")
            if min_memory > memory:
                memory = min_memory

            # Gaussian allows no decimal points.
            memory = humanize(memory, kilo=1000)

            lines = []
            lines.append("%Chk=gaussian")
            lines.append(f"%Mem={memory}")
            lines.append(f"%NProcShared={n_threads}")

            # keywords.add("FormCheck=ForceCart")
            lines.append("# " + " ".join(keywords))

            lines.append(" ")
            lines.append(f"{system.name}/{configuration.name}")
            lines.append(" ")
            lines.append(f"{configuration.charge}    {configuration.spin_multiplicity}")

            # Atoms with coordinates
            symbols = configuration.atoms.symbols
            XYZs = configuration.atoms.coordinates
            for symbol, xyz in zip(symbols, XYZs):
                x, y, z = xyz
                lines.append(f"{symbol:2}   {x:10.6f} {y:10.6f} {z:10.6f}")
            lines.append(" ")

            files = {"input.dat": "\n".join(lines)}
            logger.info("input.dat:\n" + files["input.dat"])

            exe = options["gaussian_exe"]
            exe_path = options["gaussian_path"]
            if exe_path != "":
                exe = f"{exe_path}/{exe}"

            printer.important(
                self.indent + f"    Gaussian will use {n_threads} OpenMP threads and "
                f"up to {memory} of memory.\n"
            )

            if options["gaussian_root"] != "":
                env = {"g09root": options["gaussian_root"]}
            else:
                env = {}

            if options["gaussian_environment"] != "":
                cmd = f". {options['gaussian_environment']} ; {exe}"
            else:
                cmd = exe

            cmd += " < input.dat > output.txt ; formchk gaussian.chk"

            local = seamm.ExecLocal()
            result = local.run(
                shell=True,
                cmd=cmd,
                files=files,
                env=env,
                return_files=[
                    "output.txt",
                    "gaussian.chk",
                    "gaussian.fchk",
                ],
                in_situ=True,
                directory=directory,
            )

            if result is None:
                raise RuntimeError("There was an error running Gaussian")

            # logger.debug("\n" + pprint.pformat(result))

            logger.info("stdout:\n" + result["stdout"])
            if result["stderr"] != "":
                logger.warning("stderr:\n" + result["stderr"])

        # And output
        path = directory / "output.txt"
        if path.exists():
            data = vars(cclib.io.ccread(path))
            data = self.process_data(data)
        else:
            data = {}

        # Get the data from the formatted checkpoint file
        data = self.parse_fchk(directory / "gaussian.fchk", data)

        # And parse a bit more out of the output
        if path.exists():
            data = self.parse_output(path, data)

        # Debug output
        if self.logger.isEnabledFor(logging.INFO):
            keys = "\n".join(data.keys())
            logger.info(f"Data keys:\n{keys}")
        if self.logger.isEnabledFor(logging.DEBUG):
            keys = "\n".join(data.keys())
            logger.info(f"Data keys:\n{keys}")
            logger.debug(f"Data:\n{pprint.pformat(data)}")

        # The model chemistry
        # self.model = f"{data['metadata/functional']}/{data['metadata/basis_set']}"
        if "Composite/model" in data:
            self.model = data["Composite/model"]
        else:
            self.model = (
                f"{data['metadata/methods'][-1]}/{data['method']}/"
                f"{data['metadata/basis_set']}"
            )
        logger.info(f"model = {self.model}")

        # If ran successfully, put out the success file
        if data["success"]:
            success.write_text("success")

        # Add other citations here or in the appropriate place in the code.
        # Add the bibtex to data/references.bib, and add a self.reference.cite
        # similar to the above to actually add the citation to the references.
        if "G version" in data:
            try:
                template = string.Template(self._bibliography[data["G version"]])
                citation = template.substitute(
                    month=data["G month"],
                    version=data["G revision"],
                    year=data["G year"],
                )
                self.references.cite(
                    raw=citation,
                    alias="Gaussian",
                    module="gaussian_step",
                    level=1,
                    note="The principle Gaussian citation.",
                )
            except Exception:
                pass

        return data



"""This file contains metadata describing the results from Gaussian
"""

metadata = {}

methods = {
    "DFT: Kohn-Sham density functional theory": {
        "method": "DFT",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
    },
    "HF: Hartree-Fock self consistent field (SCF)": {
        "method": "HF",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
    },
    "MP2: 2nd-order Møller–Plesset perturbation theory": {
        "method": "MP2",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "MP3: 3rd-order Møller–Plesset perturbation theory": {
        "method": "MP3",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "MP4(SDTQ): 4th-order Møller–Plesset perturbation theory": {
        "method": "MP4",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "numeric",
        "freeze core?": True,
    },
    "MP4(SDQ): 4th-order Møller–Plesset perturbation theory": {
        "method": "MP4(SDQ)",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "MP5: 5th-order Møller–Plesset perturbation theory": {
        "method": "MP5",
        "calculation": ["energy", "gradients"],
        "level": "advanced",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "CCD: coupled cluster doubles": {
        "method": "CCD",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "analytic",
        "freeze core?": True,
    },
    "CCSD: coupled cluster singles & doubles": {
        "method": "CCSD",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "numeric",
        "freeze core?": True,
    },
    "CCSD(T): coupled cluster singles & doubles plus triples": {
        "method": "CCSD(T)",
        "calculation": ["energy", "gradients"],
        "level": "normal",
        "gradients": "numeric",
        "freeze core?": True,
    },
    "CBS-4M: Complete Basis Set method of Petersson, et al.": {
        "method": "CBS-4M",
        "calculation": ["energy"],
        "level": "advanced",
        "gradients": "none",
        "nobasis": True,
    },
    "CBS-QB3: Complete Basis Set method of Petersson, et al.": {
        "method": "CBS-QB3",
        "calculation": ["energy"],
        "level": "normal",
        "gradients": "none",
        "nobasis": True,
    },
    "CBS-APNO: Complete Basis Set method of Petersson, et al.": {
        "method": "CBS-APNO",
        "calculation": ["energy"],
        "level": "advanced",
        "gradients": "none",
        "nobasis": True,
    },
    "G1: Gaussian-1 composite method": {
        "method": "G1",
        "calculation": ["energy"],
        "level": "advanced",
        "gradients": "none",
        "nobasis": True,
    },
    "G2: Gaussian-2 composite method": {
        "method": "G2",
        "calculation": ["energy"],
        "level": "advanced",
        "gradients": "none",
        "nobasis": True,
    },
    "G3: Gaussian-3 composite method": {
        "method": "G3",
        "calculation": ["energy"],
        "level": "advanced",
        "gradients": "none",
        "nobasis": True,
    },
    "G4: Gaussian-4 composite method": {
        "method": "G4",
        "calculation": ["energy"],
        "level": "advanced",
        "gradients": "none",
        "nobasis": True,
    },
}

dft_functionals = {
    "PBE : GGA of Perdew, Burke and Ernzerhof": {
        "name": "PBEPBE",
        "dispersion": ["none", "GD3BJ", "GD3", "GD2", "PFD"],
        "level": "normal",
    },
    "PBE-98 : GGA of Perdew, Burke and Ernzerhof, revised": {
        "name": "PBEhPBE",
        "dispersion": ["none", "PFD"],
        "level": "normal",
    },
    "B3LYP : hybrid functional of Becke and Lee, Yang, and Parr": {
        "name": "B3LYP",
        "dispersion": ["none", "GD3BJ", "GD3", "GD2", "PFD"],
        "level": "normal",
    },
    "HSE06 : hybrid functional of Heyd, Scuseria and Ernzerhof": {
        "name": "HSEH1PBE",
        "dispersion": ["none", "PFD"],
        "level": "normal",
    },
    "PBE0 : hybrid functional of Perdew, Burke and Ernzerhof, and Adamo": {
        "name": "PBE1PBE",
        "dispersion": ["none", "GD3BJ", "GD3", "PFD"],
        "level": "normal",
    },
}

optimization_convergence = {
    "default": "",
    "tight": "Tight",
    "very tight": "VeryTight",
    "loose": "Loose",
}

"""Properties that Gaussian produces.
`metadata["results"]` describes the results that this step can produce. It is a
dictionary where the keys are the internal names of the results within this step, and
the values are a dictionary describing the result. For example::

    metadata["results"] = {
        "total_energy": {
            "calculation": [
                "energy",
                "optimization",
            ],
            "description": "The total energy",
            "dimensionality": "scalar",
            "methods": [
                "ccsd",
                "ccsd(t)",
                "dft",
                "hf",
            ],
            "property": "total energy#QuickMin#{model}",
            "type": "float",
            "units": "E_h",
        },
    }

Fields
______

calculation : [str]
    Optional metadata describing what subtype of the step produces this result.
    The subtypes are completely arbitrary, but often they are types of calculations
    which is why this is name `calculation`. To use this, the step or a substep
    define `self._calculation` as a value. That value is used to select only the
    results with that value in this field.

description : str
    A human-readable description of the result.

dimensionality : str
    The dimensions of the data. The value can be "scalar" or an array definition
    of the form "[dim1, dim2,...]". Symmetric tringular matrices are denoted
    "triangular[n,n]". The dimensions can be integers, other scalar
    results, or standard parameters such as `n_atoms`. For example, '[3]',
    [3, n_atoms], or "triangular[n_aos, n_aos]".

methods : str
    Optional metadata like the `calculation` data. `methods` provides a second
    level of filtering, often used for the Hamiltionian for *ab initio* calculations
    where some properties may or may not be calculated depending on the type of
    theory.

property : str
    An optional definition of the property for storing this result. Must be one of
    the standard properties defined either in SEAMM or in this steps property
    metadata in `data/properties.csv`.

type : str
    The type of the data: string, integer, or float.

units : str
    Optional units for the result. If present, the value should be in these units.
"""
metadata["results"] = {
    "Virial Ratio": {
        "calculation": ["energy", "optimization", "thermodynamics", "vibrations"],
        "description": "the virial ratio",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "format": ".4f",
    },
    "Total Energy": {
        "calculation": ["energy", "optimization", "thermodynamics", "vibrations"],
        "description": "total energy including all terms",
        "dimensionality": "scalar",
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "RMS Density": {
        "calculation": ["energy", "optimization", "thermodynamics", "vibrations"],
        "description": "the RMS density difference in the SCF",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "format": ".2e",
    },
    "Geometry Optimization Converged": {
        "calculation": ["optimization"],
        "description": "whether the geometry optimzation converged",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "boolean",
        "format": "s",
    },
    "RMS Force": {
        "calculation": ["optimization"],
        "description": "the RMS force on the atoms",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "E_h/Å",
        "format": ".6f",
    },
    "Maximum Force": {
        "calculation": ["optimization"],
        "description": "the maximum force on an atom",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "E_h/Å",
        "format": ".6f",
    },
    "RMS Displacement": {
        "calculation": ["optimization"],
        "description": "the RMS displacement of the atoms",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "Å",
        "format": ".6f",
    },
    "Maximum Displacement": {
        "calculation": ["optimization"],
        "description": "the maximum displacement of an atom",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "Å",
        "format": ".6f",
    },
    "RMS Force Threshold": {
        "calculation": ["optimization"],
        "description": "the RMS force threshold for the atoms",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "E_h/Å",
        "format": ".6f",
    },
    "Maximum Force Threshold": {
        "calculation": ["optimization"],
        "description": "the maximum force threshold for an atom",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "E_h/Å",
        "format": ".6f",
    },
    "RMS Displacement Threshold": {
        "calculation": ["optimization"],
        "description": "the RMS displacement threshold for the atoms",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "Å",
        "format": ".6f",
    },
    "Maximum Displacement Threshold": {
        "calculation": ["optimization"],
        "description": "the maximum displacement threshold for an atom",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "Å",
        "format": ".6f",
    },
    "Gaussian Version": {
        "description": "the version of Gaussian used",
        "dimensionality": "scalar",
        "type": "string",
        "format": "s",
    },
    "E(gap)": {
        "calculation": ["energy", "optimization"],
        "description": (
            "The gap between the LUMO and HOMO or conduction and valence bands"
        ),
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(homo)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of highest occupied MO (HOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(homo-1)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of the next-to-highest occupied orbital (NHOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(lumo)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of lowest unoccupied MO (LUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(lumo+1)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of the second lowest unoccupied orbital (SLUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "N(homo)": {
        "calculation": ["energy", "optimization"],
        "description": "The orbital number of the HOMO (starting at 1)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "integer",
        "units": "",
        "format": "",
    },
    "Sym(homo)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the highest occupied MO (HOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "Sym(homo-1)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the next-to-highest occupied orbital (NHOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "Sym(lumo)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the lowest unoccupied MO (LUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "Sym(lumo+1)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the second lowest unoccupied orbital (SLUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "E(α-gap)": {
        "calculation": ["energy", "optimization"],
        "description": (
            "The gap between the LUMO and HOMO or conduction and valence bands"
        ),
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(α-homo)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of highest occupied MO (α-HOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(α-homo-1)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of the next-to-highest occupied orbital (α-NHOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(α-lumo)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of lowest unoccupied MO (α-LUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(α-lumo+1)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of the second lowest unoccupied orbital (α-SLUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "N(α-homo)": {
        "calculation": ["energy", "optimization"],
        "description": "The orbital number of the HOMO (starting at 1)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "integer",
        "units": "",
        "format": "",
    },
    "Sym(α-homo)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the highest occupied MO (α-HOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "Sym(α-homo-1)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the next-to-highest occupied orbital (α-NHOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "Sym(α-lumo)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the lowest unoccupied MO (α-LUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "Sym(α-lumo+1)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the second lowest unoccupied orbital (α-SLUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "E(β-gap)": {
        "calculation": ["energy", "optimization"],
        "description": (
            "The gap between the LUMO and HOMO or conduction and valence bands"
        ),
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(β-homo)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of highest occupied MO (β-HOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(β-homo-1)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of the next-to-highest occupied orbital (β-NHOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(β-lumo)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of lowest unoccupied MO (β-LUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "E(β-lumo+1)": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of the second lowest unoccupied orbital (β-SLUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "N(β-homo)": {
        "calculation": ["energy", "optimization"],
        "description": "The orbital number of the HOMO (starting at 1)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "integer",
        "units": "",
        "format": "",
    },
    "Sym(β-homo)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the highest occupied MO (β-HOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "Sym(β-homo-1)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the next-to-highest occupied orbital (β-NHOMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "Sym(β-lumo)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the lowest unoccupied MO (β-LUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "Sym(β-lumo+1)": {
        "calculation": ["symmetry", "optimization"],
        "description": "The symmetry of the second lowest unoccupied orbital (β-SLUMO)",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "atomcharges/mulliken": {
        "calculation": ["energy", "optimization"],
        "description": "The Mulliken charges of the atoms",
        "dimensionality": "[natoms]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "",
        "format": ".6f",
    },
    "coreelectrons": {
        "calculation": ["energy", "optimization"],
        "description": "The number of core electrons per atom",
        "dimensionality": "[natoms]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "integer",
        "units": "",
        "format": "",
    },
    "metadata/basis_set": {
        "description": "The basis set used in the calculation",
        "dimensionality": "scalar",
        "type": "string",
        "units": "",
        "format": "",
    },
    "metadata/cpu_time": {
        "description": "The cpu time for the calculation",
        "dimensionality": "scalar",
        "type": "string",
        "units": "",
        "format": "",
    },
    "metadata/functional": {
        "description": "The density functional",
        "dimensionality": "scalar",
        "methods": ["DFT"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "metadata/legacy_package_version": {
        "description": "The version of the Gaussian code (XXrevisionYYY)",
        "dimensionality": "scalar",
        "type": "string",
        "units": "",
        "format": "",
    },
    "metadata/methods": {
        "description": (
            "The method used in each optimization step or step of the calculation"
        ),
        "dimensionality": "[nsteps]",
        "type": "string",
        "units": "",
        "format": "",
    },
    "metadata/package": {
        "description": "The name of the code (Gaussian)",
        "dimensionality": "scalar",
        "type": "string",
        "units": "",
        "format": "",
    },
    "metadata/package_version": {
        "description": "The version of the Gaussian code (year+revision)",
        "dimensionality": "scalar",
        "type": "string",
        "units": "",
        "format": "",
    },
    "metadata/platform": {
        "description": "The platform used for the calculation",
        "dimensionality": "scalar",
        "type": "string",
        "units": "",
        "format": "",
    },
    "metadata/success": {
        "description": "Whether the calculation was successful",
        "dimensionality": "scalar",
        "type": "boolean",
        "units": "",
        "format": "",
    },
    "metadata/symmetry_detected": {
        "description": "The symmetry of the system",
        "dimensionality": "scalar",
        "type": "string",
        "units": "",
        "format": "",
    },
    "metadata/symmetry_used": {
        "description": "The symmetry used in the calculation",
        "dimensionality": "scalar",
        "type": "string",
        "units": "",
        "format": "",
    },
    "metadata/wall_time": {
        "description": "The wall-clock time for the calculation",
        "dimensionality": "scalar",
        "type": "string",
        "units": "",
        "format": "",
    },
    "moenergies": {
        "calculation": ["energy", "optimization"],
        "description": "The energy of the molecular orbitals (MOs)",
        "dimensionality": "[[nmo]*nspin]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "moments": {
        "calculation": ["energy", "optimization"],
        "description": "",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "",
        "format": ".6f",
    },
    "multipole_reference": {
        "calculation": ["energy", "optimization"],
        "description": "The reference point for the multipole expansions",
        "dimensionality": "[3]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "Å",
        "format": ".6f",
    },
    "dipole_moment": {
        "calculation": ["energy", "optimization"],
        "description": "The dipole moment",
        "dimensionality": "[3]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "debye",
        "format": ".6f",
    },
    "dipole_moment_magnitude": {
        "calculation": ["energy", "optimization"],
        "description": "The magnitude of the dipole moment",
        "dimensionality": "scalar",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "debye",
        "format": ".6f",
    },
    "quadrupole_moment": {
        "calculation": ["energy", "optimization"],
        "description": "The quadrupole moment",
        "dimensionality": "[6]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "debye*Å",
        "format": ".6f",
    },
    "octapole_moment": {
        "calculation": ["energy", "optimization"],
        "description": "The octapole moment",
        "dimensionality": "[10]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "debye*Å^2",
        "format": ".6f",
    },
    "hexadecapole_moment": {
        "calculation": ["energy", "optimization"],
        "description": "The hexadecapole moment",
        "dimensionality": "[15]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "debye*Å^3",
        "format": ".6f",
    },
    "mosysm": {
        "calculation": ["energy", "optimization"],
        "description": "The symmetry of the MOs",
        "dimensionality": "[nmo]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "string",
        "units": "",
        "format": "",
    },
    "nbasis": {
        "description": "Number of basis functions",
        "dimensionality": "scalar",
        "type": "integer",
        "units": "",
        "format": "",
    },
    "nmo": {
        "description": "Number of molecular orbitals",
        "dimensionality": "scalar",
        "type": "integer",
        "units": "",
        "format": "",
    },
    "rotconstants": {
        "calculation": ["energy", "optimization"],
        "description": "Rotational constants",
        "dimensionality": "[[3]*nsteps]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "GHz",
        "format": ".3f",
    },
    "scfenergies": {
        "calculation": ["energy", "optimization"],
        "description": "The energies per step",
        "dimensionality": "[nsteps]",
        "methods": ["DFT", "HF", "MP2", "MP3", "MP4"],
        "type": "float",
        "units": "eV",
        "format": ".3f",
    },
    "MP2 Energy": {
        "calculation": ["energy", "gradient"],
        "description": "the MP2 energy",
        "dimensionality": "scalar",
        "methods": ["MP2", "MP3", "MP4", "MP4DQ", "MP4SQD", "MP5", "CCD", "CCSD"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "MP3 Energy": {
        "calculation": ["energy", "gradient"],
        "description": "the MP3 energy",
        "dimensionality": "scalar",
        "methods": ["MP3", "MP4", "MP4DQ", "MP4SQD", "MP5", "CCD", "CCSD"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "MP4 Energy": {
        "calculation": ["energy", "gradient"],
        "description": "the MP4 energy",
        "dimensionality": "scalar",
        "methods": ["MP4", "MP4DQ", "MP4SQD", "MP5", "CCD", "CCSD"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "MP4DQ Energy": {
        "calculation": ["energy", "gradient"],
        "description": "the MP4DQ energy",
        "dimensionality": "scalar",
        "methods": ["MP4", "MP4DQ", "MP4SQD", "MP5", "CCD", "CCSD"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "MP4SDQ Energy": {
        "calculation": ["energy", "gradient"],
        "description": "the MP4SDQ energy",
        "dimensionality": "scalar",
        "methods": ["MP4", "MP4SQD", "MP5", "CCSD"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "MP5 Energy": {
        "calculation": ["energy", "gradient"],
        "description": "the MP5 energy",
        "dimensionality": "scalar",
        "methods": ["MP5"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Cluster Energy": {
        "calculation": ["energy", "gradient"],
        "description": "the CCD or CCSD energy",
        "dimensionality": "scalar",
        "methods": ["CCD", "CCSD", "CCSD(T)"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Cluster Energy with triples": {
        "calculation": ["energy", "gradient"],
        "description": "the CCSD(T) energy",
        "dimensionality": "scalar",
        "methods": ["CCSD(T)"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/Temperature": {
        "calculation": ["energy"],
        "description": "the temperature for the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO", "G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "K",
        "format": ".2f",
    },
    "Composite/Pressure": {
        "calculation": ["energy"],
        "description": "the pressure for the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO", "G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "atm",
        "format": ".2f",
    },
    "Composite/E(ZPE)": {
        "calculation": ["energy"],
        "description": "the ZPE from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO", "G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/E(Thermal)": {
        "calculation": ["energy"],
        "description": "the thermal energy from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO", "G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/E(SCF)": {
        "calculation": ["energy"],
        "description": "the SCF energy from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/DE(MP2)": {
        "calculation": ["energy"],
        "description": "dE from MP2 from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/DE(CBS)": {
        "calculation": ["energy"],
        "description": "dE from the basis extrapolation from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/DE(MP34)": {
        "calculation": ["energy"],
        "description": "dE of MP3-4 from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/DE(CCSD)": {
        "calculation": ["energy"],
        "description": "dE of CCSD from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/DE(int)": {
        "calculation": ["energy"],
        "description": "dE from internals from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/DE(empirical)": {
        "calculation": ["energy"],
        "description": "Empirical correction to the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO", "G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/E(CCSD(T))": {
        "calculation": ["energy"],
        "description": "CCSD(T) term for the Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/DE(Plus)": {
        "calculation": ["energy"],
        "description": "Plus term for the Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/DE(2DF)": {
        "calculation": ["energy"],
        "description": "The delta energy for 2D's and F term for the Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/E(Delta-G3XP)": {
        "calculation": ["energy"],
        "description": "Delta G3XP term for the Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/DE(HF)": {
        "calculation": ["energy"],
        "description": "delta E(HF) term for the Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/(0 K)": {
        "calculation": ["energy"],
        "description": "the 0 K energy from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO", "G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/Energy": {
        "calculation": ["energy"],
        "description": "the energy from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO", "G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/Enthalpy": {
        "calculation": ["energy"],
        "description": "the enthalpy from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO", "G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
    "Composite/Free Energy": {
        "calculation": ["energy"],
        "description": "the free energy from the CBS/Gn extrapolation",
        "dimensionality": "scalar",
        "methods": ["CBS-4M", "CBS-QB3", "CBS-APNO", "G1", "G2", "G3", "G4"],
        "type": "float",
        "units": "E_h",
        "format": ".6f",
    },
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `gaussian_step` package."""

import pytest  # noqa: F401
import gaussian_step  # noqa: F401


def test_construction():
    """Just create an object and test its type."""
    result = gaussian_step.Gaussian()
    assert str(type(result)) == "<class 'gaussian_step.gaussian.Gaussian'>"
